package com.supervision.manage.controller.diseasemanage;

import com.supervision.manage.service.DiseaseAncillaryManageService;
import com.supervision.model.DiseaseAncillary;
import com.supervision.vo.ask.AncillaryItemReqVo;
import com.supervision.vo.ask.ConfigAncillaryItemResVO;
import com.supervision.vo.manage.DiseaseAncillaryReqVo;
import com.supervision.vo.manage.DiseaseAncillaryResVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.List;


@Api(tags = "疾病辅助检查管理")
@RestController
@RequestMapping("diseaseAncillary")
@RequiredArgsConstructor
public class DiseaseAncillaryManageController {

   private final DiseaseAncillaryManageService diseaseAncillaryManageService;

    @ApiOperation("新增辅助检查信息")
    @PostMapping("/save")
    public String saveAncillary(@RequestBody DiseaseAncillaryReqVo diseaseAncillary) {

        diseaseAncillary.floatDiagnosticCriteria();
        DiseaseAncillary result = diseaseAncillaryManageService.saveAncillary(diseaseAncillary);
        return result.getId();

    }


    @ApiOperation("删除疾病辅助检查信息")
    @DeleteMapping("/delete")
    public boolean deleteDiseaseAncillary(@RequestParam("id") String id) {

        return diseaseAncillaryManageService.deleteDiseaseAncillary(id);

    }


    @ApiOperation("修改辅助检查信息")
    @PutMapping("/update")
    public boolean updateAncillary(@RequestBody DiseaseAncillaryReqVo diseaseAncillary) {

        diseaseAncillary.floatDiagnosticCriteria();
        return diseaseAncillaryManageService.updateAncillary(diseaseAncillary);

    }

    @ApiOperation("根据疾病id查询疾病辅助检查信息列表")
    @GetMapping("/queryListByDiseaseId")
    public List<DiseaseAncillaryResVo> queryListByDiseaseId(@RequestParam("diseaseId") String diseaseId) {

        return diseaseAncillaryManageService.queryListByDiseaseId(diseaseId);

    }

    @ApiOperation("查询疾病辅助检查项目列表")
    @GetMapping("queryAncillaryItemList")
    public List<ConfigAncillaryItemResVO> queryAncillaryItemList(AncillaryItemReqVo ancillaryItemReqVo) {
        return diseaseAncillaryManageService.queryAncillaryItemList(ancillaryItemReqVo);
    }

}
