package com.supervision.manage.controller.medicalrec;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.supervision.dto.CommonDictTreeDTO;
import com.supervision.manage.pojo.vo.MedicalRecCheckResVO;
import com.supervision.manage.pojo.vo.MedicalRecInfoVO;
import com.supervision.manage.pojo.vo.MedicalRecManageVO;
import com.supervision.manage.pojo.vo.MedicalRecQaVO;
import com.supervision.service.CommonDicService;
import com.supervision.vo.manage.DiseaseAncillaryResVo;
import com.supervision.vo.manage.DiseasePhysicalResVo;
import com.supervision.vo.manage.DiseaseTreatmentPlanResVo;
import com.supervision.vo.manage.MedicalRecPageResVO;
import com.supervision.manage.service.MedicalRecManageService;
import com.supervision.model.Disease;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotBlank;
import java.util.List;

@Api(tags = "病历管理")
@RestController
@RequestMapping("medicalRecManage")
@RequiredArgsConstructor
public class MedicalRecManageController {

    private final MedicalRecManageService medicalRecManageService;

    @ApiOperation("初步诊断下拉列表联想(这里目前只查单一疾病)")
    @GetMapping("queryDiseaseListByDropList")
    public List<Disease> queryDiseaseListByDropList() {
        return medicalRecManageService.queryDiseaseListByDropList();
    }

    @ApiOperation("病历管理分页查询")
    @GetMapping("queryMedicalRecPage")
    public IPage<MedicalRecPageResVO> queryMedicalRecManagePage(@ApiParam("主诉") String selfDescKeyword,
                                                                @ApiParam("性别") String gender,
                                                                @ApiParam("疾病ID") String diseaseId,
                                                                Integer pageNum, Integer pageSize) {
        return medicalRecManageService.queryMedicalRecManagePage(selfDescKeyword, gender, diseaseId, pageNum, pageSize);
    }

    @ApiOperation("创建病历")
    @PostMapping("createMedicalRec")
    public void createMedicalRec(@RequestBody MedicalRecManageVO reqVO) {
        medicalRecManageService.createMedicalRec(reqVO);
    }

    @ApiOperation("创建疾病时可选的的疾病列表")
    @GetMapping("queryDiseaseListByCreat")
    public List<Disease> queryDiseaseListByCreat() {
        return medicalRecManageService.queryDiseaseListByCreat();
    }

    @ApiOperation("创建病历时查看配置的辅助检查项")
    @GetMapping("queryDiseaseAncillaryByCreat")
    public List<DiseaseAncillaryResVo> queryDiseaseAncillaryByCreat(String diseaseId){
        return medicalRecManageService.queryDiseaseAncillaryByCreat(diseaseId);
    }

    @ApiOperation("创建病历时查看配置的体格检查项")
    @GetMapping("queryDiseasePhysicalByCreat")
    public List<DiseasePhysicalResVo> queryDiseasePhysicalByCreat(String diseaseId){
        return medicalRecManageService.queryDiseasePhysicalByCreat(diseaseId);
    }

    @ApiOperation("创建病历时查看配置的处置计划")
    @GetMapping("queryDiseaseTreatmentPlanByCreat")
    public List<DiseaseTreatmentPlanResVo> queryDiseaseTreatmentPlanByCreat(String diseaseId){
        return medicalRecManageService.queryDiseaseTreatmentPlanByCreat(diseaseId);
    }

    @ApiOperation("创建病历时,根据疾病ID查询配置的问题(目前只支持单一疾病)")
    @GetMapping("queryQuestionListByCreat")
    public List<MedicalRecQaVO> queryQuestionListByCreat(String diseaseId) {
        return medicalRecManageService.queryQuestionListByCreat(diseaseId);
    }


    @ApiOperation("修改病历")
    @PostMapping("modifyMedicalRec")
    public void modifyMedicalRec(@RequestBody MedicalRecManageVO reqVO) {
        medicalRecManageService.modifyMedicalRec(reqVO);
    }

    @ApiOperation("查询病例详细信息")
    @GetMapping("queryMedicalRecInfo")
    public MedicalRecInfoVO queryMedicalRecInfo(String id) {
        return medicalRecManageService.queryMedicalRecInfo(id);
    }

    @ApiOperation("删除病历")
    @GetMapping("deleteMedicalRec")
    public void deleteMedicalRec(String id) {
        medicalRecManageService.deleteMedicalRec(id);
    }



}
