package com.supervision.manage.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import com.supervision.manage.service.DiseasePhysicalManageService;
import com.supervision.manage.service.PhysicalLocationManageService;
import com.supervision.manage.service.PhysicalToolManageService;
import com.supervision.model.ConfigPhysicalLocation;
import com.supervision.model.DiseaseAncillary;
import com.supervision.model.DiseasePhysical;
import com.supervision.service.ConfigPhysicalLocationService;
import com.supervision.service.DiseasePhysicalService;
import com.supervision.vo.manage.DiseasePhysicalLocationNodeVo;
import com.supervision.vo.manage.DiseasePhysicalResVo;
import com.supervision.vo.manage.PhysicalLocationNode;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

@Service
@RequiredArgsConstructor
public class DiseasePhysicalManageServiceImpl implements DiseasePhysicalManageService {

    private final DiseasePhysicalService diseasePhysicalService;

    private final PhysicalLocationManageService physicalToolManageService;


    @Override
    public List<DiseasePhysicalResVo> queryListByDiseaseId(String diseaseId) {

        Assert.isTrue(StrUtil.isNotEmpty(diseaseId),"id不能为空");
        List<DiseasePhysicalResVo> diseasePhysicalResVos = diseasePhysicalService.queryListByDiseaseId(diseaseId);

        diseasePhysicalResVos.forEach(DiseasePhysicalResVo::populateOther);
        return diseasePhysicalResVos;
    }

    @Override
    public DiseasePhysical savePhysical(DiseasePhysical diseasePhysical) {

        assertDiseasePhysical(diseasePhysical);

        diseasePhysicalService.save(diseasePhysical);

        return diseasePhysical;
    }

    @Override
    public boolean deletePhysical(String id) {
        Assert.isTrue(StrUtil.isNotEmpty(id),"id不能为空");

        return diseasePhysicalService.removeById(id);
    }

    @Override
    public boolean updatePhysical(DiseasePhysical diseasePhysical) {
        Assert.isTrue(StrUtil.isNotEmpty(diseasePhysical.getDiseaseId()),"疾病id不能为空");

        return diseasePhysicalService.updateById(diseasePhysical);
    }

    @Override
    public boolean deleteByDiseaseId(String diseaseId) {

        Assert.notEmpty(diseaseId,"疾病id不能为空");
        return diseasePhysicalService.lambdaUpdate().eq(DiseasePhysical::getDiseaseId,diseaseId).remove();
    }

    @Override
    public List<DiseasePhysicalLocationNodeVo> queryTree(String diseaseId, String toolId) {

        // 1. 查询体格检查位置树
        List<PhysicalLocationNode> physicalLocationNodes = physicalToolManageService.queryTree();
        if (CollectionUtil.isEmpty(physicalLocationNodes)){
            return CollectionUtil.newArrayList();
        }

        PhysicalLocationNode physicalLocationNode = new PhysicalLocationNode();
        physicalLocationNode.setChild(physicalLocationNodes);
        DiseasePhysicalLocationNodeVo bean = BeanUtil.toBean(physicalLocationNode, DiseasePhysicalLocationNodeVo.class);

        if (StrUtil.isEmpty(toolId)){
            return bean.getChild();
        }
        // 2. 根据疾病id查询疾病体格检查项
        List<DiseasePhysicalResVo> diseasePhysicalResVos = diseasePhysicalService.queryListByDiseaseId(diseaseId);
        if (CollectionUtil.isEmpty(diseasePhysicalResVos)){
            return bean.getChild();
        }

        // 3. 填充疾病体格检查树中的flag属性
        List<String> locationIds = diseasePhysicalResVos.stream().filter(vo->toolId.equals(vo.getToolId()))
                .map(DiseasePhysical::getLocationId).distinct().collect(Collectors.toList());
        populateDiseasePhysicalLocationNodeFlag(bean,locationIds);

        return bean.getChild();
    }


    private void populateDiseasePhysicalLocationNodeFlag(DiseasePhysicalLocationNodeVo node,List<String> locationIds) {
        if (StrUtil.isNotEmpty(node.getId())){
            node.setFlag(locationIds.contains(node.getId()));
        }
        if (CollectionUtil.isNotEmpty(node.getChild())){
            for (DiseasePhysicalLocationNodeVo childNode : node.getChild()) {
                populateDiseasePhysicalLocationNodeFlag(childNode,locationIds);
            }
        }
    }

    private void assertDiseasePhysical(DiseasePhysical diseasePhysical){

        Assert.isTrue(StrUtil.isNotEmpty(diseasePhysical.getDiseaseId()),"疾病id不能为空");

        Assert.notNull(diseasePhysical.getPrimarilyDiagnosisCriteriaFlag(),"初步诊断依据不能为空");

        Assert.notNull(diseasePhysical.getBasisConfirmFlag(),"证实诊断依据不能为空");

        Assert.notNull(diseasePhysical.getBasisIdentificationFlag(),"鉴别诊断依据不能为空");

        Assert.notNull(diseasePhysical.getFullCheckFlag(),"全面诊断不能为空");

        Assert.notNull(diseasePhysical.getDiagnosisAssessmentFlag(),"判断不能为空");

        Assert.notNull(diseasePhysical.getRequireCheckFlag(),"是否必查不能为空");

        Assert.notNull(diseasePhysical.getDiagnosisAssessmentFlag(),"判断不能为空");

        Assert.notNull(diseasePhysical.getExpectedDiagnosisResult(),"预期诊断结果不能为空");

    }

}
