package com.supervision.manage.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.supervision.manage.service.AskQuestionLibraryManageService;
import com.supervision.manage.service.DiseaseAncillaryManageService;
import com.supervision.manage.service.DiseaseQuestionManageService;
import com.supervision.model.DiseaseQuestion;
import com.supervision.service.AskTemplateQuestionLibraryService;
import com.supervision.service.DiseaseQuestionService;
import com.supervision.vo.manage.AskQuestionLibraryReqVo;
import com.supervision.vo.manage.AskQuestionLibraryResVo;
import com.supervision.vo.manage.DiseaseQuestionResVo;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
@RequiredArgsConstructor
public class DiseaseQuestionManageServiceImpl implements DiseaseQuestionManageService {

    private final DiseaseQuestionService diseaseQuestionService;

    private final AskQuestionLibraryManageService askQuestionLibraryManageService;

    @Override
    public List<DiseaseQuestionResVo> queryPageByDiseaseId(String diseaseId) {

        Assert.notEmpty(diseaseId,"diseaseId 不允许为空");

        return diseaseQuestionService.queryPageByDiseaseId(diseaseId);
    }

    @Override
    public void batchSaveQuestion(List<DiseaseQuestion> diseaseQuestionList) {

        Assert.notEmpty(diseaseQuestionList,"数据为空");
        diseaseQuestionList.forEach(this::assertDiseaseQuestion);
        Assert.isTrue(CollectionUtil.isEmpty(findListByDiseaseIdAndQuestionIdGroup(diseaseQuestionList)),"数据已存在");

        diseaseQuestionService.saveBatch(diseaseQuestionList);

    }

    @Override
    public boolean deleteDiseaseQuestion(String id) {

        Assert.notEmpty(id,"id不允许为空");

       return diseaseQuestionService.removeById(id);
    }

    @Override
    public boolean deleteByDiseaseId(String diseaseId) {
        Assert.notEmpty(diseaseId,"疾病id不允许为空");

        return diseaseQuestionService.lambdaUpdate().eq(DiseaseQuestion::getDiseaseId, diseaseId).remove();
    }

    @Override
    public Page<AskQuestionLibraryResVo> queryQaLibPageList(Integer pageNum, Integer pageSize, AskQuestionLibraryReqVo askQuestionLibraryReqVo) {

        return diseaseQuestionService.queryQaLibPageList(pageNum,pageSize,askQuestionLibraryReqVo);
    }


    private List<DiseaseQuestion> findListByDiseaseIdAndQuestionIdGroup(List<DiseaseQuestion> diseaseQuestionList){

        if (CollectionUtil.isEmpty(diseaseQuestionList)){
            return CollectionUtil.newArrayList();
        }

        LambdaQueryChainWrapper<DiseaseQuestion> query = diseaseQuestionService.lambdaQuery();
        diseaseQuestionList.forEach(diseaseQuestion -> query
                .or(i -> i.eq(DiseaseQuestion::getQuestionId, diseaseQuestion.getQuestionId())
                        .and(i1 -> i1.eq(DiseaseQuestion::getDiseaseId, diseaseQuestion.getDiseaseId()))));

        return query.list();

    }


    private void assertDiseaseQuestion(DiseaseQuestion diseaseQuestion){

        Assert.isTrue(StrUtil.isEmpty(diseaseQuestion.getId()),"id需要为空");

        Assert.notEmpty(diseaseQuestion.getDiseaseId(),"疾病id不能为空");

        Assert.notEmpty(diseaseQuestion.getQuestionId(),"问题id不能为空");
    }

}
