package com.supervision.manage.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import com.supervision.manage.service.ConfigTreatmentPlanManageService;
import com.supervision.manage.service.DiseaseTreatmentPlanManageService;
import com.supervision.model.DiseaseTreatmentPlan;
import com.supervision.service.DiseaseTreatmentPlanService;
import com.supervision.vo.manage.DiseaseTreatmentPlanResVo;
import com.supervision.vo.manage.DiseaseTreatmentPlanTreeNode;
import com.supervision.vo.manage.TreatmentPlanTreeNode;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

@Service
@RequiredArgsConstructor
public class DiseaseTreatmentPlanManageServiceImpl implements DiseaseTreatmentPlanManageService {

    private final DiseaseTreatmentPlanService diseaseTreatmentPlanService;

    private final ConfigTreatmentPlanManageService configTreatmentPlanManageService;

    @Override
    public List<DiseaseTreatmentPlanResVo> queryListByDiseaseId(String diseaseId) {

        Assert.notEmpty(diseaseId,"疾病id不能为空");
        return diseaseTreatmentPlanService.queryListByDiseaseId(diseaseId);
    }

    @Override
    public DiseaseTreatmentPlan saveTreatmentPlan(DiseaseTreatmentPlan diseaseTreatmentPlan) {

        assertDiseaseTreatmentPlan(diseaseTreatmentPlan);

        diseaseTreatmentPlanService.save(diseaseTreatmentPlan);

        return diseaseTreatmentPlan;

    }

    @Override
    public void batchSaveTreatmentPlan(List<DiseaseTreatmentPlan> diseaseTreatmentPlanList) {
        Assert.notEmpty(diseaseTreatmentPlanList,"数据不能为空");
        diseaseTreatmentPlanList.forEach(this::assertDiseaseTreatmentPlan);

        diseaseTreatmentPlanService.saveBatch(diseaseTreatmentPlanList);
    }

    @Override
    public boolean deleteDiseaseTreatmentPlan(String id) {
        Assert.notEmpty(id,"id不能为空");

        return diseaseTreatmentPlanService.removeById(id);
    }

    @Override
    public boolean deleteByDiseaseId(String diseaseId) {
        Assert.notEmpty(diseaseId,"疾病id不能为空");

        return diseaseTreatmentPlanService.lambdaUpdate().eq(DiseaseTreatmentPlan::getDiseaseId,diseaseId).remove();
    }

    @Override
    public List<DiseaseTreatmentPlanTreeNode> queryDiseaseTreatmentPlanTree(Integer disposalMethod, String diseaseId) {

        // 1. 查询处置计划树
        List<TreatmentPlanTreeNode> treatmentPlanTreeNodes = configTreatmentPlanManageService.queryTree(disposalMethod);
        if (CollUtil.isEmpty(treatmentPlanTreeNodes)){
            return CollectionUtil.newArrayList();
        }

        TreatmentPlanTreeNode treatmentPlanTreeNode = new TreatmentPlanTreeNode();
        treatmentPlanTreeNode.setChild(treatmentPlanTreeNodes);
        DiseaseTreatmentPlanTreeNode rootNode = new DiseaseTreatmentPlanTreeNode(treatmentPlanTreeNode);

        // 2.根据疾病id查询疾病处置计划信息
        List<DiseaseTreatmentPlanResVo> diseaseTreatmentPlanResVos = this.queryListByDiseaseId(diseaseId);

        // 3. 初始化flag
        if (CollUtil.isNotEmpty(diseaseTreatmentPlanResVos)){
            rootNode.initFlag(diseaseTreatmentPlanResVos.stream().map(DiseaseTreatmentPlanResVo::getPlanId).collect(Collectors.toList()));
        }

        return rootNode.getChild();
    }



    private void assertDiseaseTreatmentPlan(DiseaseTreatmentPlan diseaseTreatmentPlan){

        Assert.isTrue(StrUtil.isEmpty(diseaseTreatmentPlan.getId()),"id需要为空");

        Assert.notEmpty(diseaseTreatmentPlan.getDiseaseId(),"疾病id不能为空");

        Assert.notEmpty(diseaseTreatmentPlan.getPlanId(),"处置计划id不能为空");
    }
}
