package com.supervision.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.supervision.dto.CommonDictTreeDTO;
import com.supervision.model.CommonDic;
import com.supervision.service.CommonDicService;
import com.supervision.mapper.CommonDicMapper;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author Administrator
 * @description 针对表【vp_common_dic(通用字典表)】的数据库操作Service实现
 * @createDate 2023-11-23 10:52:22
 */
@Service
public class CommonDicServiceImpl extends ServiceImpl<CommonDicMapper, CommonDic>
        implements CommonDicService {

    @Override
    public List<CommonDictTreeDTO> queryCommonDictTree(String groupCode) {
        List<CommonDic> list = super.lambdaQuery().eq(CommonDic::getGroupCode, groupCode).list();
        Map<Long, List<CommonDic>> commonDicParentIdMap = list.stream().filter(e -> ObjectUtil.isNotNull(e.getParentId()))
                .collect(Collectors.groupingBy(CommonDic::getParentId));
        // 首先构建第一层节点
        List<CommonDictTreeDTO> topList = list.stream().filter(e -> ObjectUtil.isNull(e.getParentId()))
                .map(node -> BeanUtil.toBean(node, CommonDictTreeDTO.class)).collect(Collectors.toList());
        // 递归构建树
        recursionBuildTree(commonDicParentIdMap, topList);
        return topList;
    }

    private void recursionBuildTree(Map<Long, List<CommonDic>> commonDicParentIdMap, List<CommonDictTreeDTO> sourceList) {
        for (CommonDictTreeDTO parentNode : sourceList) {
            List<CommonDic> commonDicList = commonDicParentIdMap.getOrDefault(parentNode.getId(), new ArrayList<>());
            List<CommonDictTreeDTO> childNodeList = new ArrayList<>();

            for (CommonDic commonDic : commonDicList) {
                CommonDictTreeDTO childNode = BeanUtil.toBean(commonDic, CommonDictTreeDTO.class);
                childNodeList.add(childNode);
            }
            parentNode.setChildDictTreeList(childNodeList);
            // 继续迭代生成子集
            recursionBuildTree(commonDicParentIdMap, childNodeList);
        }
    }

}




