package com.supervision.rasa.service.impl;

import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONUtil;
import com.supervision.exception.BusinessException;
import com.supervision.model.RasaModelInfo;
import com.supervision.rasa.pojo.dto.RasaReqDTO;
import com.supervision.rasa.pojo.dto.RasaResDTO;
import com.supervision.vo.rasa.RasaTalkVo;
import com.supervision.rasa.service.RasaTalkService;
import com.supervision.service.RasaModeService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;
@Service
@Slf4j
@RequiredArgsConstructor
public class RasaTalkServiceImpl implements RasaTalkService {

    @Value("${rasa.url}")
    private String rasaUrl;

    private final RasaModeService rasaModeService;
    @Override
    public List<String> talkRasa(RasaTalkVo rasaTalkVo)  {

        if (StrUtil.isEmpty(rasaTalkVo.getModelId())){
            throw new BusinessException("modelId is not allow empty ");
        }
        RasaModelInfo rasaModelInfo = rasaModeService.queryByModelId("1");

        RasaReqDTO rasaReqDTO = new RasaReqDTO();
        rasaReqDTO.setSender(rasaTalkVo.getSessionId());
        rasaReqDTO.setMessage(rasaTalkVo.getQuestion());

        String rasaUrl = getRasaUrl(rasaModelInfo.getPort());
        log.info("talkRasa: url is: {}",rasaUrl);

        String post = HttpUtil.post(rasaUrl, JSONUtil.toJsonStr(rasaReqDTO));

        List<RasaResDTO> list = JSONUtil.toList(post, RasaResDTO.class);

        return list.stream().map(RasaResDTO::getText).collect(Collectors.toList());
    }

    private String getRasaUrl(int port){

        return StrUtil.format(rasaUrl, port);
    }
}
