package com.supervision.controller;

import com.supervision.model.ConfigAncillaryItem;
import com.supervision.pojo.vo.AskAncillaryBatchReqVO;
import com.supervision.vo.ask.AncillaryItemReqVo;
import com.supervision.vo.ask.AskAncillaryHistoryResVO;
import com.supervision.pojo.vo.AskAncillaryResultReqVO;
import com.supervision.vo.ask.ConfigAncillaryItemResVO;
import com.supervision.service.AskAncillaryService;
import com.supervision.vo.ask.DiagnosisAncillaryRecordVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

@Api(tags = "辅助检查")
@RestController
@RequestMapping("askAncillary")
@RequiredArgsConstructor
public class AskAncillaryController {

    private final AskAncillaryService askAncillaryService;

    @ApiOperation("查询辅助检查项目列表")
    @GetMapping("queryAncillaryItemList")
    public List<ConfigAncillaryItemResVO> queryAncillaryItemList(AncillaryItemReqVo ancillaryItemReqVo) {
        return askAncillaryService.queryAncillaryItemList(ancillaryItemReqVo);
    }

    @ApiOperation("查询辅助检查项目目录树")
    @GetMapping("queryAncillaryItemTree")
    public Map<String, List<ConfigAncillaryItem>> queryAncillaryItemTree(){
        return askAncillaryService.queryAncillaryItemTree();
    }

    @ApiOperation("批量进行辅助检查")
    @PostMapping("execAskAncillaryBatch")
    public void execAskAncillaryBatch(@RequestBody List<AskAncillaryBatchReqVO> reqVO){
        askAncillaryService.execAskAncillaryBatch(reqVO);
    }

    @Deprecated
    @ApiOperation("进行辅助检查,并获取结果")
    @GetMapping("queryAskAncillaryResult")
    public DiagnosisAncillaryRecordVo execAskAncillaryResult(AskAncillaryResultReqVO reqVO) {
        return askAncillaryService.execAskAncillaryResult(reqVO);
    }

    @ApiOperation("查询辅助检查的历史记录")
    @GetMapping("queryAskAncillaryHistory")
    public List<AskAncillaryHistoryResVO> queryAskAncillaryHistory(String processId) {
        return askAncillaryService.queryAskAncillaryHistory(processId);
    }


}
