package com.supervision.controller;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.supervision.constant.UserTokenConstant;
import com.supervision.usermanage.UserResourceCheck;
import com.supervision.util.SpringBeanUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import javax.websocket.*;
import javax.websocket.server.PathParam;
import javax.websocket.server.ServerEndpoint;
import java.io.IOException;
import java.util.concurrent.ConcurrentHashMap;

@Component
@Slf4j
@ServerEndpoint("/webSocket/{uid}")
public class WebSocketServer {

    private static RedisTemplate<String, String> redisTemplate ;

    //concurrent包的线程安全Set，用来存放每个客户端对应的WebSocketServer对象。
    public static final ConcurrentHashMap<String, Session> SESSION_POOL = new ConcurrentHashMap<>();

    private static UserResourceCheck userResourceCheck ;

    // 因为是ServerEndpoint是多例,所以需要这样注入
    @Autowired
    public void setUserResourceCheck(UserResourceCheck userResourceCheck) {
        WebSocketServer.userResourceCheck = userResourceCheck;
    }
    // 因为是ServerEndpoint是多例,所以需要这样注入
    @Autowired
    public void setRedisTemplate(RedisTemplate<String, String> redisTemplate) {
        WebSocketServer.redisTemplate = redisTemplate;
    }

    /**
     * 有客户端连接成功
     */
    @OnOpen
    public void onOpen(Session session, @PathParam(value = "uid") String uid) {
        userResourceCheck.achieveDiagnoseResourceAndOpenConnection(uid, session);
        SESSION_POOL.put(uid, session);
        log.info("用户:{}登录成功", uid);
    }

    /**
     * 连接关闭调用的方法
     */
    @OnClose
    public void onClose(Session session, @PathParam(value = "uid") String uid) {
        redisTemplate.opsForHash().delete(UserTokenConstant.USER_WEBSOCKET_CACHE, uid, session.getId());
        SESSION_POOL.remove(uid);
        log.info("用户:{}关闭，从Redis中移除,当前连接数为：{}", uid, redisTemplate.opsForHash().size(UserTokenConstant.USER_WEBSOCKET_CACHE));
    }

    /**
     * 发生错误
     */
    @OnError
    public void onError(Session session, @PathParam(value = "uid") String uid, Throwable throwable) {
        redisTemplate.opsForHash().delete(UserTokenConstant.USER_WEBSOCKET_CACHE, uid, session.getId());
        SESSION_POOL.remove(uid);
        log.error("用户:{}发生错误，从Redis中移除,当前连接数为：{}", uid, redisTemplate.opsForHash().size(UserTokenConstant.USER_WEBSOCKET_CACHE), throwable);
    }


}
