package com.supervision.service.impl;

import cn.hutool.core.lang.Assert;
import com.supervision.exception.BusinessException;
import com.supervision.model.MedicalRec;
import com.supervision.model.Process;
import com.supervision.model.User;
import com.supervision.service.AskProcessService;
import com.supervision.service.MedicalRecService;
import com.supervision.service.ProcessService;
import com.supervision.util.SequenceUtil;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;

@Service
@RequiredArgsConstructor
public class AskProcessServiceImpl implements AskProcessService {

    private final ProcessService processService;

    private final MedicalRecService medicalRecService;

    @Override
    public Process creatDiagnosisProcess(String patientId,String medicalId, User user) {
        MedicalRec medicalRec = Optional.ofNullable(medicalRecService.getById(medicalId)).orElseThrow(() -> new BusinessException("未找到病历"));
        Process process = new Process();
        process.setPatientId(patientId);
        process.setMedicalRecId(medicalId);
        process.setDiseaseId(medicalRec.getDiseaseId());
        process.setUserId(user.getId());
        process.setStatus(0);
        process.setCreateUserId(user.getId());
        process.setUpdateUserId(user.getId());
        process.setProcessType(0);//默认学习类型
        process.setProcessNo(SequenceUtil.getProcessNo());
        processService.save(process);
        return process;
    }

    @Override
    public List<Process> queryUserProcess(String userId) {
        return processService.lambdaQuery().eq(Process::getUserId,userId).list();
    }

    @Override
    public boolean deleteDiagnosisProcess(String processId) {
        Assert.notEmpty(processId,"问诊实例不能为空");

        return processService.removeById(processId);
    }
}
