package com.supervision.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.supervision.model.Process;
import com.supervision.model.ProcessMedical;
import com.supervision.model.User;
import com.supervision.service.*;
import com.supervision.vo.ask.*;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;


@Service
@RequiredArgsConstructor
@Slf4j
public class MedicalRecordServiceImpl implements MedicalRecordService {

    private final ProcessMedicalService processMedicalService;

    private final ProcessService processService;

    private final DiagnosisPrimaryService diagnosisPrimaryService;

    private final DiagnosisPhysicalRecordService diagnosisPhysicalRecordService;

    private final DiagnosisAncillaryRecordService diagnosisAncillaryRecordService;

    private final UserService userService;

    @Override
    public boolean saveMedicalRecord(ProcessMedical processMedical) {

        saveAssert(processMedical);

        return processMedicalService.save(processMedical);
    }

    @Override
    public ProcessMedicalResVo findByProcessId(String processId) {

        Assert.notEmpty(processId,"流程id不允许为空");

        return processMedicalService.findByProcessId(processId);
    }

    @Override
    public boolean saveOrUpdateMedicalRecord(ProcessMedical processMedical) {
        log.info(JSONUtil.toJsonStr(processMedical));

        Assert.notEmpty(processMedical.getProcessId(),"流程id不为空");
        List<ProcessMedical> list = processMedicalService.lambdaQuery().eq(ProcessMedical::getProcessId, processMedical.getProcessId()).list();
        if (CollUtil.isEmpty(list)){
            // 新增
            return saveMedicalRecord(processMedical);
        }

        // 修改
        processMedical.setId(CollUtil.getFirst(list).getId());
        return processMedicalService.updateById(processMedical);
    }

    @Override
    public ProcessMedicalViewDetailResVo queryViewDetail(String processId) {

        Assert.notEmpty(processId,"流程id不能为空");

        // 1.获取电子病例基本信息
        ProcessMedicalResVo processMedicalResVo = this.findByProcessId(processId);
        if (Objects.isNull(processMedicalResVo)){
            return null;
        }
        ProcessMedicalViewDetailResVo resultVo = BeanUtil.toBean(processMedicalResVo, ProcessMedicalViewDetailResVo.class);

        // 2. 获取初步诊断疾病信息
        List<DiagnosisPrimaryVO> diagnosisPrimaryVOS = diagnosisPrimaryService.queryAskPrimaryList(processId);
        resultVo.setPrimaryDiseaseNameList(
                diagnosisPrimaryVOS.stream().map(DiagnosisPrimaryVO::getPrimaryDiseaseName).distinct().collect(Collectors.toList()));

        // 3. 获取检查项目信息
        //  3.1 体格检查
        List<AskPhysicalHistoryResVO> askPhysicalHistoryResVOS = diagnosisPhysicalRecordService.queryAskPhysicalHistory(processId);
        resultVo.setPhysicalDiagnosisNameList(
                askPhysicalHistoryResVOS.stream().map(AskPhysicalHistoryResVO::getToolName).distinct().collect(Collectors.toList()));
        //  3.2 辅助检查
        List<AskAncillaryHistoryResVO> askAncillaryHistoryResVOS = diagnosisAncillaryRecordService.queryAncillaryRecord(processId);
        resultVo.setAncillaryDiagnosisNameList(
                askAncillaryHistoryResVOS.stream().map(AskAncillaryHistoryResVO::getItemName).distinct().collect(Collectors.toList()));

        // 4. 获取用户姓名
        User user = userService.getById(processMedicalResVo.getUserId());
        if (Objects.nonNull(user)){
            resultVo.setUserName(user.getName());
        }

        return resultVo;
    }

    private void saveAssert(ProcessMedical processMedical){
        Assert.notEmpty(processMedical.getProcessId(),"流程id不为空");

        Process process = processService.getById(processMedical.getProcessId());
        Assert.notNull(process,"流程数据不存在");

        processMedical.setMedicalRecId(process.getMedicalRecId());
        processMedical.setPatientId(process.getPatientId());

        Integer count = processMedicalService.lambdaQuery().eq(ProcessMedical::getProcessId, processMedical.getProcessId()).count();
        Assert.isFalse(count > 0,"当前实例已经存在电子病例信息");
    }
}
